function [f,fit,tree_comp,dis]=fitness(ind,ind_opt,u,y,yr,uv,yv,yrv,beta,lambda,FitnessGauss,comm,par,sigmamax)
% [f,fit,tree_comp]=fitness(ind,ind_opt,u,y,yr,uv,yv,yrv,tr,beta,lambda,FitnessGauss,comm,par)
% Usage: This function computes the fitness of an individual in the population.
%        f = the fitness function with penalties
%        fit = the fit of the model without the running average
%        tree_comp = complexity of the model
%        dis = discrepancy
%        ind = the gene
%        ind_opt = the optimized gene (with parameters)
%        u/uv = log10(tau) vector (v=validation set)
%        y/yv = Im part (v=validation set)
%        yr/yrv = Re part (v=validation set)
%        tr = complexity of the last generation best model
%        beta = guessed number of functions/peaks
%        lambda = actual number of functions/peaks
%        FitnessGauss = normalization factor
%        sigmahalf = width factor of all range
%        width,halfwidth parameters for the width penalty
% Written by: Alex Tesler
% Modified by: Yehu-David Horev, Arthur Doroshev
%--------------------------------------------------------------------------
% filter
% load the filter parameters
% load omega.GPDAT 
omega=load([comm.dirname,'\','omega.gpdat'],'-ascii');
w0=omega(1);
w1=omega(2);

% transpose u (log10(tau)) back into omega
w=1./(10.^u);

% calculate filter vector
filter=(1+exp(-5.*log10(w./w0)./log10(w1/w0))).^(-1);

% Defining independent variable vectors.
% in case there is a mistake & no varaiable is sent
if isempty(ind_opt)==1
    f=0;
    fit=0;
    % send in an empty fitness
    return
end
%--------------------------------------------------------------------------
% input data:
%[num_data,nu]=size(u);
uu=u(1:end,:); % u=log10(tau)
yy=y(1:end,:); % y=Im
yyr=yr(1:end,:); % yr=Re

% Computing model prediction:
tau_diff=par.tau_diff;
hh=-(u(2)-u(1))/tau_diff;
t=(-20:hh:20); 
ind_opt=strrep(ind_opt,'u1','t');
ym=eval(ind_opt);

% Convolution:
[y_r,yi]=int_simpson_matrix(ym,uu,hh);

% modified aveY/aveYr which contains 2 averages in each decade 26-03-2014
freq=w./(2*pi);

% 0.5 with 10 points in decade results in averaging every 5 points
num_p=floor(0.5*floor(length(freq)/(log10(freq(end)/freq(1))))); 
aveY=wgt_ave(yy,num_p);
aveYr=wgt_ave(yyr,num_p);

% Computing correlation coefficient:
SSY1=(yy-aveY).*filter;
SSYr1=(yyr-aveYr).*filter;

%SSY=sqrt(sum((SSY1).^2)./(num_data)); % SSTO of imag part
SSY=rms(SSY1);
%SSYr=sqrt(sum((SSYr1).^2)./(num_data)); % SSTO of real part
SSYr=rms(SSYr1);

SSF1=(yi-yy).*filter;
SSFr1=(y_r-yyr).*filter;

%SSF=sqrt(sum(((SSF1).^2))./(num_data)); % SSE of imag part
SSF=rms(SSF1);
%SSFr=sqrt(sum(((SSFr1).^2))./(num_data)); % SSE of real part
SSFr=rms(SSFr1);

fit1i=SSY/(SSY+SSF);
fit1r=SSYr/(SSYr+SSFr);

alpha=par.alpha;
fit1=alpha*fit1i+(1-alpha)*fit1r;
dis1=rms(SSFr1+1i.*SSF1); 

% in case there is a mistake & no varaiable is sent
if isreal(fit1)==0
    % send in an empty fitness
    ff=0; 
    fit1=0;
end
%--------------------------------------------------------------------------
% input data 2nd set:
uu=uv(1:end,:); % u=log10(tau)
yy=yv(1:end,:); % y=Im
yyr=yrv(1:end,:); % yr=Re

% transpose u (log10(tau)) back into omega
w=1./(10.^uu);

% calculate filter vector
filter=(1+exp(-5.*log10(w./w0)./log10(w1/w0))).^(-1);

% Computing model prediction:
hh=-(uu(2)-uu(1))/tau_diff;
t=(-20:hh:20); 
ind_opt=strrep(ind_opt,'u1','t');
ym=eval(ind_opt);

% Convolution:
[y_r1,yi1]=int_simpson_matrix(ym,uu,hh);

% modified aveY/aveYr which contains 2 averages in each decade 26-03-2014
freq=w./(2*pi);

% 0.5 with 10 points in decade results in averaging every 5 points
num_p=floor(0.5*floor(length(freq)/(log10(freq(end)/freq(1))))); 
aveY=wgt_ave(yy,num_p);
aveYr=wgt_ave(yyr,num_p);

% Computing correlation coefficient:
SSY1=(yy-aveY).*filter;
SSYr1=(yyr-aveYr).*filter;

%SSY=sqrt(sum((SSY1).^2)./(num_data)); %SSTO of imag part
SSY=rms(SSY1);
%SSYr=sqrt(sum((SSYr1).^2)./(num_data)); %SSTO of real part
SSYr=rms(SSYr1);
    
SSF1=(yi1-yy).*filter;
SSFr1=(y_r1-yyr).*filter;

%SSF=sqrt(sum(((SSF1).^2))./(num_data)); %SSE of imag part
SSF=rms(SSF1);
%SSFr=sqrt(sum(((SSFr1).^2))./(num_data)); %SSE of real part
SSFr=rms(SSFr1);
    
fit2i=SSY/(SSY+SSF);
fit2r=SSYr/(SSYr+SSFr);

fit2=alpha*fit2i+(1-alpha)*fit2r;
dis2=rms(SSFr1+1i.*SSF1);

% in case there is a mistake & no varaiable is sent
if isreal(fit2)==0
    ff=0; 
    fit2=0;
end
%----------------------------------------------------------------------
% Calculating width for merit function:
sigmot=Pickswidth(ind_opt); %was with (ind_opt,hh)
sigma=max(sigmot);

% Computing model complexity (counting the number of tree leaves):
tree_comp=treecomp(ind);

% Calculating Area for merit function:
t=-20:hh:u(1); 
Area=trapz(hh,eval(ind_opt));
t=-20:hh:20; 

% Computing final competability & discrepancy:
fit=(0.8*fit1+0.2*fit2);
dis=(0.8*dis1+0.2*dis2);

% calculation of the complexity sigmoid (4.5) with changes by Yoed, Shany & Yehu:
f=fit*((1+1/3*exp(2*(tree_comp/lambda-6)))^-1)*((1+exp(2*(lambda-beta-2.5)))^-1)*((1+exp(-7*(sigmamax-sigma)/sigmamax))^-1)*((1-FitnessGauss)+FitnessGauss*exp(-(1-Area)^2));
% f=(the fitness)*(penalty for to high # of free parameters for each peak)*(penalty for high # of peaks)*(penalty for wide peaks)*(penalty for area other than 1)

% in case there's a mistake:
f(isnan(f))=0;
f(isinf(f))=0;

% in case there's a mistake:
fit(isinf(fit))=inf;
fit(isnan(fit))=inf;